@@ -142,7 +142,24 @@ selectable_changed_cb(SexyUrlLabel *url_label)
 	SexyUrlLabelPrivate *priv = SEXY_URL_LABEL_GET_PRIVATE(url_label);
 
 	if (priv->event_window != NULL)
+	{
+		GdkCursor *cursor = NULL;
+
+		if (gtk_label_get_selectable(GTK_LABEL(url_label)) &&
+			priv->active_link == NULL)
+			cursor = gdk_cursor_new_for_display(
+				gtk_widget_get_display(GTK_WIDGET(url_label)), GDK_XTERM);
+		gdk_window_set_cursor(priv->event_window, cursor);
+
+		if (cursor)
+			gdk_cursor_unref(cursor);
+
+		/*
+		 * GtkLabel recreates its event window when the selectable property
+		 * changes, which will cover ours.
+		 */
 		gdk_window_raise(priv->event_window);
+	}
 }
 
 static void
@@ -295,7 +312,13 @@ sexy_url_label_leave_notify_event(GtkWidget *widget, GdkEventCrossing *event)
 
 	if (event->mode == GDK_CROSSING_NORMAL)
 	{
-		gdk_window_set_cursor(priv->event_window, NULL);
+		GdkCursor *cursor = NULL;
+		if (gtk_label_get_selectable(GTK_LABEL(widget)))
+			cursor = gdk_cursor_new_for_display(
+				gtk_widget_get_display(widget), GDK_XTERM);
+		gdk_window_set_cursor(priv->event_window, cursor);
+		if (cursor)
+			gdk_cursor_unref(cursor);
 		priv->active_link = NULL;
 	}
 
@@ -384,10 +407,19 @@ sexy_url_label_realize(GtkWidget *widget)
 							  GDK_LEAVE_NOTIFY_MASK);
 	attributes_mask = GDK_WA_X | GDK_WA_Y;
 
+	if (gtk_label_get_selectable(GTK_LABEL(widget))) {
+		attributes.cursor = gdk_cursor_new_for_display(
+			gtk_widget_get_display(widget), GDK_XTERM);
+		attributes_mask |= GDK_WA_CURSOR;
+	}
+
 	priv->event_window =
 		gdk_window_new(gtk_widget_get_parent_window(widget), &attributes,
 					   attributes_mask);
 	gdk_window_set_user_data(priv->event_window, widget);
+
+	if (attributes_mask & GDK_WA_CURSOR)
+		gdk_cursor_unref (attributes.cursor);
 }
 
 static void
@@ -427,7 +459,7 @@ sexy_url_label_unmap(GtkWidget *widget)
 	if (priv->event_window != NULL)
 		gdk_window_hide(priv->event_window);
 
-	GTK_WIDGET_CLASS(parent_class)->map(widget);
+	GTK_WIDGET_CLASS(parent_class)->unmap(widget);
 }
 
 static void
@@ -701,7 +733,7 @@ text_handler(GMarkupParseContext *context,
 	SexyUrlLabel *url_label   = SEXY_URL_LABEL(user_data);
 	SexyUrlLabelPrivate *priv = SEXY_URL_LABEL_GET_PRIVATE(url_label);
 
-	gchar *newtext = g_markup_escape_text (text, text_len);
+	gchar *newtext = g_markup_escape_text(text, text_len);
 	g_string_append_len(priv->temp_markup_result, newtext, strlen (newtext));
 	g_free (newtext);
 }
@@ -832,3 +864,5 @@ sexy_url_label_set_markup(SexyUrlLabel *url_label, const gchar *markup)
 
 	update_wrap_width(url_label, priv->wrap_width);
 }
+
+// vim:ts=4 sw=4
